@@ -15,6 +15,8 @@ References:
 
 # === LICENSE =================================================================
 # TODO
+
+
 #------------------------------------------------------------------------------
 # TODO:
 # - make stream optional in PptUnexpectedData
@@ -22,14 +24,16 @@ References:
 # - license
 # - make buffered stream from output of iterative_decompress
 # - maybe can merge the 2 decorators into 1? (with_opened_main_stream)
-#
+
+
 # CHANGELOG:
 # 2016-05-04 v0.01 CH: - start parsing "Current User" stream
 # 2016-07-20 v0.50 SL: - added Python 3 support
 # 2016-09-13       PL: - fixed olefile import for Python 2+3
 #                      - fixed format strings for Python 2.6 (issue #75)
+# 2017-04-23 v0.51 PL: - fixed absolute imports and issue #101
 
-__version__ = '0.50'
+__version__ = '0.51'
 
 
 # --- IMPORTS ------------------------------------------------------------------
@@ -39,16 +43,22 @@ import logging
 import struct
 import traceback
 import os
-
-try:
-    # absolute import when oletools is installed
-    import oletools.thirdparty.olefile as olefile
-except:
-    # relative import otherwise
-    import thirdparty.olefile as olefile
-
 import zlib
 
+# IMPORTANT: it should be possible to run oletools directly as scripts
+# in any directory without installing them with pip or setup.py.
+# In that case, relative imports are NOT usable.
+# And to enable Python 2+3 compatibility, we need to use absolute imports,
+# so we add the oletools parent folder to sys.path (absolute+normalized path):
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if not _parent_dir in sys.path:
+    sys.path.insert(0, _parent_dir)
+
+from oletools.thirdparty.olefile import olefile
+
 
 # a global logger object used for debugging:
 log = olefile.get_logger('ppt')
@@ -1130,7 +1140,7 @@ class PptParser(object):
             log.debug('using open OleFileIO')
             self.ole = ole
         else:
-            log.debug('Opening file ' + ole)
+            log.debug('Opening file {0}'.format(ole))
             self.ole = olefile.OleFileIO(ole)
 
         self.fast_fail = fast_fail
