@@ -17,7 +17,7 @@ http://www.decalage.info/python/oletools
 
 #=== LICENSE =================================================================
 
-# rtfobj is copyright (c) 2012-2016, Philippe Lagadec (http://www.decalage.info)
+# rtfobj is copyright (c) 2012-2017, Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -64,8 +64,17 @@ http://www.decalage.info/python/oletools
 # 2016-08-08       PL: - added option -s to save objects to files
 # 2016-08-09       PL: - fixed issue #78, improved regex
 # 2016-09-06       PL: - fixed issue #83, backward compatible API
+# 2016-11-17 v0.51 PL: - updated call to oleobj.OleNativeStream
+# 2017-03-12       PL: - fixed imports for Python 2+3
+#                      - fixed hex decoding bug in RtfObjParser (issue #103)
+# 2017-03-29       PL: - fixed RtfParser to handle issue #152 (control word with
+#                        long parameter)
+# 2017-04-11       PL: - added detection of the OLE2Link vulnerability CVE-2017-0199
+# 2017-05-04       PL: - fixed issue #164 to handle linked OLE objects
+# 2017-06-08       PL: - fixed issue/PR #143: bin object with negative length
+# 2017-06-29       PL: - temporary fix for issue #178
 
-__version__ = '0.50'
+__version__ = '0.51'
 
 # ------------------------------------------------------------------------------
 # TODO:
@@ -82,12 +91,22 @@ __version__ = '0.50'
 import re, os, sys, binascii, logging, optparse
 import os.path
 
-from thirdparty.xglob import xglob
-from oleobj import OleObject, OleNativeStream
-import oleobj
-
-from thirdparty.tablestream import tablestream
+# IMPORTANT: it should be possible to run oletools directly as scripts
+# in any directory without installing them with pip or setup.py.
+# In that case, relative imports are NOT usable.
+# And to enable Python 2+3 compatibility, we need to use absolute imports,
+# so we add the oletools parent folder to sys.path (absolute+normalized path):
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if not _parent_dir in sys.path:
+    sys.path.insert(0, _parent_dir)
 
+from oletools.thirdparty.xglob import xglob
+from oletools.thirdparty.tablestream import tablestream
+from oletools.oleobj import OleObject, OleNativeStream
+from oletools import oleobj
 
 # === LOGGING =================================================================
 
@@ -173,8 +192,8 @@ ASCII_NAME = b'([a-zA-Z]{1,250})'
 # SIGNED_INTEGER = r'(-?\d{1,250})'
 SIGNED_INTEGER = b'(-?\\d+)'
 
-# Note for issue #78: need to match "\A-" not followed by digits
-CONTROL_WORD = b'(?:\\\\' + ASCII_NAME + b'(?:' + SIGNED_INTEGER + b'(?=[^0-9])|(?=[^a-zA-Z0-9])))'
+# Note for issue #78: need to match "\A-" not followed by digits (or the end of string)
+CONTROL_WORD = b'(?:\\\\' + ASCII_NAME + b'(?:' + SIGNED_INTEGER + b'(?=[^0-9])|(?=[^a-zA-Z0-9])|$))'
 
 re_control_word = re.compile(CONTROL_WORD)
 
@@ -261,7 +280,10 @@ DESTINATION_CONTROL_WORDS = frozenset((
     b"mzeroAsc", b"mzeroDesc", b"mzeroWid", b"nesttableprops", b"nexctfile", b"nonesttables", b"objalias", b"objclass",
     b"objdata", b"object", b"objname", b"objsect", b"objtime", b"oldcprops", b"oldpprops", b"oldsprops", b"oldtprops",
     b"oleclsid", b"operator", b"panose", b"password", b"passwordhash", b"pgp", b"pgptbl", b"picprop", b"pict", b"pn", b"pnseclvl",
-    b"pntext", b"pntxta", b"pntxtb", b"printim", b"private", b"propname", b"protend", b"protstart", b"protusertbl", b"pxe",
+    b"pntext", b"pntxta", b"pntxtb", b"printim",
+    # It seems \private should not be treated as a destination (issue #178)
+    # b"private",
+    b"propname", b"protend", b"protstart", b"protusertbl", b"pxe",
     b"result", b"revtbl", b"revtim", b"rsidtbl", b"rtf", b"rxe", b"shp", b"shpgrp", b"shpinst", b"shppict", b"shprslt", b"shptxt",
     b"sn", b"sp", b"staticval", b"stylesheet", b"subject", b"sv", b"svb", b"tc", b"template", b"themedata", b"title", b"txe", b"ud",
     b"upr", b"userprops", b"wgrffmtfilter", b"windowcaption", b"writereservation", b"writereservhash", b"xe", b"xform",
@@ -310,10 +332,16 @@ class Destination(object):
 
 class RtfParser(object):
     """
-    Very simple generic RTF parser
+    Very simple but robust generic RTF parser, designed to handle
+    malformed malicious RTF as MS Word does
     """
 
     def __init__(self, data):
+        """
+        RtfParser constructor.
+        
+        :param data: bytes object containing the RTF data to be parsed 
+        """
         self.data = data
         self.index = 0
         self.size = len(data)
@@ -324,18 +352,38 @@ class RtfParser(object):
         self.current_destination = document_destination
 
     def parse(self):
+        """
+        Parse the RTF data
+        
+        :return: nothing
+        """
+        # Start at beginning of data
         self.index = 0
+        # Loop until the end
         while self.index < self.size:
             if self.data[self.index] == BRACE_OPEN:
+                # Found an opening brace "{": Start of a group
                 self._open_group()
                 self.index += 1
                 continue
             if self.data[self.index] == BRACE_CLOSE:
+                # Found a closing brace "}": End of a group
                 self._close_group()
                 self.index += 1
                 continue
             if self.data[self.index] == BACKSLASH:
-                m = re_control_word.match(self.data, self.index)
+                # Found a backslash "\": Start of a control word or control symbol
+                # Use a regex to extract the control word name if present:
+                # NOTE: the full length of the control word + its optional integer parameter
+                # is limited by MS Word at 253 characters, so we have to run the regex
+                # on a cropped string:
+                data_cropped = self.data[self.index:]
+                if len(data_cropped)>253:
+                    data_cropped = data_cropped[:254]
+                # append a space so that the regex can check the following character:
+                data_cropped += b' '
+                # m = re_control_word.match(self.data, self.index, self.index+253)
+                m = re_control_word.match(data_cropped)
                 if m:
                     cword = m.group(1)
                     param = None
@@ -348,11 +396,14 @@ class RtfParser(object):
                     if cword == b'bin':
                         self._bin(m, param)
                     continue
+                # Otherwise, it may be a control symbol:
                 m = re_control_symbol.match(self.data, self.index)
                 if m:
                     self.control_symbol(m)
                     self.index += len(m.group())
                     continue
+            # Otherwise, this is plain text:
+            # Use a regex to match all characters until the next brace or backslash:
             m = re_text.match(self.data, self.index)
             if m:
                 self._text(m)
@@ -429,6 +480,8 @@ class RtfParser(object):
 
     def _control_word(self, matchobject, cword, param):
         #log.debug('control word %r at index %Xh' % (matchobject.group(), self.index))
+        # TODO: according to RTF specs v1.9.1, "Destination changes are legal only immediately after an opening brace ({)"
+        # (not counting the special control symbol \*, of course)
         if cword in DESTINATION_CONTROL_WORDS:
             # log.debug('%r is a destination control word: starting a new destination' % cword)
             self._open_destination(matchobject, cword)
@@ -453,6 +506,12 @@ class RtfParser(object):
 
     def _bin(self, matchobject, param):
         binlen = int(param)
+        if binlen < 0:
+            log.warn('Detected anti-analysis trick: \\bin object with negative length at index %X' % self.index)
+            # binlen = int(param.strip('-'))
+            # According to my tests, if the bin length is negative,
+            # it should be treated as a null length:
+            binlen=0
         log.debug('\\bin: reading %d bytes of binary data' % binlen)
         # TODO: handle optional space?
         # TODO: handle negative length, and length greater than data
@@ -516,6 +575,7 @@ class RtfObjParser(RtfParser):
         self.objects = []
 
     def open_destination(self, destination):
+        # TODO: detect when the destination is within an objdata, report as obfuscation
         if destination.cword == b'objdata':
             log.debug('*** Start object data at index %Xh' % destination.start)
 
@@ -529,10 +589,10 @@ class RtfObjParser(RtfParser):
             # Filter out all whitespaces first (just ignored):
             hexdata1 = destination.data.translate(None, b' \t\r\n\f\v')
             # Then filter out any other non-hex character:
-            hexdata = re.sub(b'[^a-hA-H0-9]', b'', hexdata1)
+            hexdata = re.sub(b'[^a-fA-F0-9]', b'', hexdata1)
             if len(hexdata) < len(hexdata1):
                 # this is only for debugging:
-                nonhex = re.sub(b'[a-hA-H0-9]', b'', hexdata1)
+                nonhex = re.sub(b'[a-fA-F0-9]', b'', hexdata1)
                 log.debug('Found non-hex chars in hexdata: %r' % nonhex)
             # MS Word accepts an extra hex digit, so we need to trim it if present:
             if len(hexdata) & 1:
@@ -552,7 +612,7 @@ class RtfObjParser(RtfParser):
                 rtfobj.oledata = obj.data
                 rtfobj.is_ole = True
                 if obj.class_name.lower() == 'package':
-                    opkg = OleNativeStream(bindata=obj.data)
+                    opkg = OleNativeStream(bindata=obj.data, package=True)
                     rtfobj.filename = opkg.filename
                     rtfobj.src_path = opkg.src_path
                     rtfobj.temp_path = opkg.temp_path
@@ -572,6 +632,7 @@ class RtfObjParser(RtfParser):
         # TODO: extract useful cwords such as objclass
         # TODO: keep track of cwords inside objdata, because it is unusual and indicates potential obfuscation
         # TODO: same with control symbols, and opening bracket
+        log.debug('- Control word "%s", param=%s, level=%d' % (cword, param, self.group_level))
         pass
 
 
@@ -648,11 +709,22 @@ def process_file(container, filename, data, output_dir=None, save_object=False):
     rtfp = RtfObjParser(data)
     rtfp.parse()
     for rtfobj in rtfp.objects:
+        ole_color = None
         pkg_color = None
         if rtfobj.is_ole:
-            ole_column = 'format_id: %d\n' % rtfobj.format_id
+            ole_column = 'format_id: %d ' % rtfobj.format_id
+            if rtfobj.format_id == oleobj.OleObject.TYPE_EMBEDDED:
+                ole_column += '(Embedded)\n'
+            elif rtfobj.format_id == oleobj.OleObject.TYPE_LINKED:
+                ole_column += '(Linked)\n'
+            else:
+                ole_column += '(Unknown)\n'
             ole_column += 'class name: %r\n' % rtfobj.class_name
-            ole_column += 'data size: %d' % rtfobj.oledata_size
+            # if the object is linked and not embedded, data_size=None:
+            if rtfobj.oledata_size is None:
+                ole_column += 'data size: N/A'
+            else:
+                ole_column += 'data size: %d' % rtfobj.oledata_size
             if rtfobj.is_package:
                 pkg_column = 'Filename: %r\n' % rtfobj.filename
                 pkg_column += 'Source path: %r\n' % rtfobj.src_path
@@ -666,6 +738,11 @@ def process_file(container, filename, data, output_dir=None, save_object=False):
                     pkg_column += '\nEXECUTABLE FILE'
             else:
                 pkg_column = 'Not an OLE Package'
+            # Detect OLE2Link exploit
+            # http://www.kb.cert.org/vuls/id/921560
+            if rtfobj.class_name == 'OLE2Link':
+                ole_color = 'red'
+                ole_column += '\nPossibly an exploit for the OLE2Link vulnerability (VU#921560, CVE-2017-0199)'
         else:
             pkg_column = ''
             ole_column = 'Not a well-formed OLE object'
@@ -675,7 +752,7 @@ def process_file(container, filename, data, output_dir=None, save_object=False):
             '%08Xh' % rtfobj.start,
             ole_column,
             pkg_column
-            ), colors=(None, None, None, pkg_color)
+            ), colors=(None, None, ole_color, pkg_color)
         )
         tstream.write_sep()
     if save_object:
@@ -702,7 +779,8 @@ def process_file(container, filename, data, output_dir=None, save_object=False):
                     fname = '%s_object_%08X.noname' % (fname_prefix, rtfobj.start)
                 print('  saving to file %s' % fname)
                 open(fname, 'wb').write(rtfobj.olepkgdata)
-            elif rtfobj.is_ole:
+            # When format_id=TYPE_LINKED, oledata_size=None
+            elif rtfobj.is_ole and rtfobj.oledata_size is not None:
                 print('Saving file embedded in OLE object #%d:' % i)
                 print('  format_id  = %d' % rtfobj.format_id)
                 print('  class name = %r' % rtfobj.class_name)
@@ -788,7 +866,7 @@ def main():
                         format='%(levelname)-8s %(message)s')
     # enable logging in the modules:
     log.setLevel(logging.NOTSET)
-    oleobj.log.setLevel(logging.NOTSET)
+    oleobj.enable_logging()
 
     for container, filename, data in xglob.iter_files(args, recursive=options.recursive,
         zip_password=options.zip_password, zip_fname=options.zip_fname):
